/*This file is a part of luscus project*/
/*Licensed under the Academic Free License version 3.0*/
#include<gtk/gtk.h>
#include<gdk/gdkkeysyms.h>
#include"luscus.h"
#include"gv.h"
#include"gv_functions.h"
#include "center.xpm"

static gboolean hovering_over_link = FALSE;
static GdkCursor *hand_cursor = NULL;
static GdkCursor *regular_cursor = NULL;

void do_link(GtkTextBuffer *buffer, GtkTextIter *iter, gchar *text, gint page)
{
  GtkTextTag *tag;

  tag = gtk_text_buffer_create_tag(buffer, NULL, "foreground", "blue", "underline", PANGO_UNDERLINE_SINGLE, NULL);
  g_object_set_data(G_OBJECT(tag), "page", GINT_TO_POINTER(page));
  gtk_text_buffer_insert_with_tags(buffer, iter, text, -1, tag, NULL);
}

void show_page(GtkTextBuffer *buffer, gint page)
{
  GtkTextIter iter;
  GtkTextTag *tag;
  GdkPixbuf *pixbuf;

  pixbuf = gdk_pixbuf_new_from_xpm_data(centr_xpm);

  gtk_text_buffer_set_text (buffer, "", 0);
  gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);

  if (page == 1)
  {
    do_link(buffer, &iter, "Overview\n\n", 2);
    do_link(buffer, &iter, "How-to\n\n", 3);
    do_link(buffer, &iter, "Hotkeys\n\n", 4);
  }
  else if (page == 2) /*Overview*/
  {
    gtk_text_buffer_insert(buffer, &iter,
    "Luscus Grid/Geometry Viewer/Editor is an OpenGL based code for visualization of molecular orbitals and densities, and visualization of molecular structures (with the possibility to manipulate them on the screen), and visualization of some properties.\nLuscus can operate with different kinds of files: grid files (usually with extension grid) (generated by the program GRID_IT), XYZ files and molden files (generated by MOLCAS).\n", -1);
    do_link(buffer, &iter, "Go back", 1);
  }
  else if (page == 3) /*How-to*/
  {
    do_link(buffer, &iter, "Add new fragment to the molecule\n", 5);
    do_link(buffer, &iter, "Add or remove atom\n", 6);
    do_link(buffer, &iter, "Add or remove dummy atom\n", 7);
    do_link(buffer, &iter, "Analize symmetry or apply symmetry operations to the molecule\n", 8);
    do_link(buffer, &iter, "Change atom\n", 9);
    do_link(buffer, &iter, "Change atom colors, valencies and atomic radii\n", 10);
    do_link(buffer, &iter, "Change background color\n", 11);
    do_link(buffer, &iter, "Change orbital color\n", 12);
    do_link(buffer, &iter, "Change orbital type\n", 17);
    do_link(buffer, &iter, "Change lightning angle\n", 13);
    do_link(buffer, &iter, "Display geometries/orbitals from the same angle\n", 14);
    do_link(buffer, &iter, "Position molecule in the center-of-mass\n", 15);
    do_link(buffer, &iter, "Move fragment in the molecule\n\n", 16);

    do_link(buffer, &iter, "Go back", 1);
  }
  else if (page == 4) /*Hotkeys*/
  {
    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "General hotkeys\n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "<ctrl>Q			quit\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F10				Exit (exit implies possible back up of edited files).\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Escape			unset different editing modes \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F1				Display the help screen (with a list of hot-keys)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "q/Q				decrease/increase quality of drawing\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F2				save INPORB file or XYZ file\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F9				Save current settings\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "<shift>+F9		Edit colors for background, labels, orbitals and extra planes.\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "r/R/g/G/b/B		Interactively change RGB code for the background, labels, and orbitals (selected by Shift-F9 key)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F5				Save screenshot in PostScript/povray format\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "P				Print screen in PostScript (level 2) format. \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "p				Save povray file.\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "z/Z				Zoom the grid or the molecule\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "x				Maximize the screen \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "v/V				Create/restore .ViewPoint file (viewing conditions)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Home			Move the molecule to the center of the screen\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Up/Down/Left/Right	Move the position of the picture on screen (with shift - make bigger steps) \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "l				Switch to/from the mode: move the light position with the mouse.\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "w				Switch between greyscale and colored picture\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "m/Z			Start/stop animation\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "W				Switch to Tee time mode\n\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Grid Mode\n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "+/-				increase/decrease the isosurface value (the step and initial value can be modified by command line parameters) \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Pg up/pg down		display next/previous orbital. In multiview mode (F3 has been pressed), use magnify glass. \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "t/T				change transparency level\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "f/i/1/2(a)/3/s/d		change the type of the current orbital to frozen, inactive, RAS1, RAS2, RAS3, secondary, deleted\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Space/middle mouse			change the type of the current orbital (by loop)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F2				save INPORB file (file will be saved as filename.GvOrb)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F3				Switch to/from multiview mode. In multiview mode the orbital type is shown by different backgrounds (rainbow colors). User can modify the type of orbitals, by pressing middle mouse button (or Space).\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F4				Enter an isovalue, or an orbital number, or create a filter (for more information check tutorial for gv)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Delete			Hide the orbital from the list\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Insert			Restore all hidden orbitals\n\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Coordinate Mode\n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "F2				Save coordinates (file will be saved as filename000.xyz) \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "<shift>F2		Save coordinates (overwrite the file) \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "left mouse		select an atom by clicking on it\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "<shift>left mouse		mark an atom \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F3				display menu with fragments\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "M				if Mopac is installed - optimize geometry (you might need to modify sbin/runmopac script and fix the location of Mopac installation)\n\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys if no atoms are selected \n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "+/-				change a size of atoms and bonds\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Insert			add an atom, or last inserted fragment \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "End				add dummy atoms (reference points) on the direction of axis\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "a				Mark hydrogen atoms in the molecule \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F8				analyze the symmetry of the molecule and display symmetry elements\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Delete			delete dummy atoms\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Home			Move the molecule to the center of screen\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "*				Reverse the selection\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "#				if several atoms are marked, sort them, and place into the beginning of xyz file\n\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Selection mode (1 atom is selected) \n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "Space/middle mouse	remove selection\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F3				display a menu with fragments, to be inserted close to selected atom\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Insert			add an atom (or last selected fragment) near selected atom\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Delete			delete selected atom \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Home		 	place the origin to the position of selected atom\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Pg up/Pg down	Change selected atom to one from the list (H,C,N,O,F,S,Cl)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F4 or = 			invoke an edit box, where you can type an element name for selected atom\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F7				Mark atoms connected to the selected atom\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "a				Mark all atoms which are the same elements as selected\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F8				apply Inversion symmetry around selected atom\n\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Selection mode for bond (2 atoms are selected)\n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "Space/middle mouse	remove selection\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Insert			create a bond between selected atoms\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Delete			delete the bond between selected atoms\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Pg up/Pg down	change the type of the bond between atoms\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "+/-				change the distance between atoms. Note that first selected atom (blue) will move\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F4 or =			invoke an edit box, where you can type an interatomic distance \n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F6				Watch the value of selected bond\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F7				Mark all connected atoms around the first atom in a selected bond into a group\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F8				apply C2 symmetry around an axis specified by selected atoms\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "#				Change the order of selected atoms\n\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Selection mode for angle (3 atoms are selected)\n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "Space/middle mouse	remove selection\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "+/-				change the angle between selected atoms. Note that first selected atom (blue) will move\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Pg up/Pg down	change the angle according to 'standard' angle values (by loop)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F4 or =			invoke an edit box, where you can type a dihedral angle value (or, you may type the value directly)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F8				apply mirror symmetry around an plain specified by selected atoms\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F6				Watch the value of selected angle\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Selection mode for dihedral angle (4 atoms are selected)\n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "Space/middle mouse	remove selection\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "+/-				change the dihedral angle between selected atoms. Note that first selected atom (blue) will move\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F4 or =			invoke an edit box, where you can type a dihedral angle value (or, you may type the value directly)\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "F6				Watch the value of selected angle\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Molden mode for a orbital file (e.g. scf.molden) \n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "Pg up/Pg down	Display charges\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Molden mode for a frequency file\n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "F3				draw graphical information in a separate window\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "+/-				change the speed of vibrations\n", -1);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "Hot keys in Molden mode for a geometry file\n\n", -1, tag, NULL);

    tag = gtk_text_buffer_create_tag(buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_insert_with_tags(buffer, &iter, "keyword		description\n", -1, tag, NULL);

    gtk_text_buffer_insert(buffer, &iter, "F3				draw graphical information in a separate window\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Home			show initial structure\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "End				show the resulting structure\n", -1);

    do_link(buffer, &iter, "Go back", 1);
  }
  else if (page == 5)
  {
    /*Add new fragment to the molecule*/
    gtk_text_buffer_insert(buffer, &iter, "Select an atom, you wish to connect with new fragment by clicking on it with left mouse button.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Open the \"fragments\" notebook menu and select the fragment you wish to insert.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Once the fragment is inserted, you can repeat inserting that fragment with the \"ADD\" button from the \"edit\" notebook menu.\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);   
  }
  else if (page == 6)
  { /*Add or remove atom*/
    gtk_text_buffer_insert(buffer, &iter, "Open \"edit\" notebook menu and press \"Add\" button.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "By default the hydrogen atom will be inserted. You can change the atom by clicking the \"Change\" button from the \"edit\" notebook menu. Please note; the atom, you wish to change must be the only selected atom!\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "If you inserted a molecular fragment previously, you must select \"H\" from the \"frgments\" notebook menu.\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 7)
  { /*Add or remove dummy atom*/
    gtk_text_buffer_insert(buffer, &iter, "Open \"edit\" notebook menu and press \"Add\" button.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Change the atom by clicking the \"Change\" button and select \"XX\"\n\n\n", -1);

    gtk_text_buffer_insert(buffer, &iter, "If you wish to insert dummy atom between two atoms, open \"edit\" notebook menu and find \"dummy atoms\" section.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Select two atoms and click on the \"Add\" atom in the \"dummy atoms\" section.\n\n\n", -1);

    gtk_text_buffer_insert(buffer, &iter, "You can add dummy atoms at the coordinate axis by clicking \"Add\" button in the \"dummy atoms\" section on the \"edit\" notebook menu when no atom is selected.\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 8)
  { /*Analize symmetry or apply symmetry operations to the molecule*/
    gtk_text_buffer_insert(buffer, &iter, "To analyze symmetry of the molecule, first unselect all atoms. (Use \"Unselect\" button from the \"edit\" notebook panel.)\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Use the \"Symmetry\" button from the \"Symmetry\" notebook page.\n\n\n", -1);

    gtk_text_buffer_insert(buffer, &iter, "To apply inversion of symmetry, select one atom and use the \"Symmetry\" button from the \"Symmetry\" notebook page.\n\n\n", -1);

    gtk_text_buffer_insert(buffer, &iter, "To apply C2 rotation, select two atoms and use the \"Symmetry\" button from the \"Symmetry\" notebook page.\n\n\n", -1);

    gtk_text_buffer_insert(buffer, &iter, "To apply mirror symmetry, select three atoms and use the \"Symmetry\" button from the \"Symmetry\" notebook page.\n\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 9)
  { /*Change atom*/
    gtk_text_buffer_insert(buffer, &iter, "Select an atom, you wish to change by clicking on it with left mouse button.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Open \"edit\" notebook menu and click on the \"Change\" button.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "The \"Choose element\" dialog will appear from which you can select a element.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Alternatively, you can change basic atom properties by clicking on the \"custom data\" expander at the bottom of the \"Choose element\" dialog. Please note, if you use this option, properties of all atoms of the same element will be chaged. This change will be just for the currant session of the program. If you wish to make permanent change, please use \"Edit->Atom properties\" dialog from the menubar\n\n", -1);

    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 10)
  { /*Change atom colors, valencies and atomic radii*/
    gtk_text_buffer_insert(buffer, &iter, "If you wish to change atom data during the curret session of the program:\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "	Select an atom, you wish to change by clicking on it with left mouse button.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "	Open \"edit\" notebook menu and click on the \"Change\" button.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "	Clicking on the \"custom data\" expander at the bottom of the \"Choose element\" dialog.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "If you wish make a permanent change to the atom data:\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "	Use \"Edit->Atom properties\" dialog from the menubar\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 11)
  {/*Change background color*/
    gtk_text_buffer_insert(buffer, &iter, "Click on the Edit->Background color menu item\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 12)
  {/*Change orbital color*/
    gtk_text_buffer_insert(buffer, &iter, "Click on the Edit->Orbital negative color or Edit->Orbital positive color menu item\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 13)
  {/*Change lightning angle*/
    gtk_text_buffer_insert(buffer, &iter, "Click on the View->Move light menu item\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Click on the display area and drag mouse as if you are rotating molecule. The light position should be changed.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "If you are satisfied with the new lightning position click on the View->Move light menu item again.\n\n", -1);

    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 14)
  {/*Display orbitals from the same angle*/
    gtk_text_buffer_insert(buffer, &iter, "Click on the View->Create viewpoint menu item.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Select new orbital or new geometry of open new file.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Click on the View->Restore viewpoint menu item. The molecule (orbital) will be displayed at the same angle as it was when the \"Create viewpoint\" was clicked.\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 15)
  {/*Position molecule in the center-of-mass*/
    gtk_text_buffer_insert(buffer, &iter, "Select the \"general\" notebook page\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Click the  icon ", -1);
    gtk_text_buffer_insert_pixbuf (buffer, &iter, pixbuf);
    gtk_text_buffer_insert(buffer, &iter, "\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 16)
  {/*Move fragment in the molecule*/
    gtk_text_buffer_insert(buffer, &iter, "Select the \"edit\" notebook page\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Enter the marking mode by pressing the \"mark\" button.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Mark atoms in the fragment you wish to move.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Exit the marking mode by pressing the \"mark\" button again.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Select two or more atoms in the molecule by clicking on them.\n\n", -1);
    gtk_text_buffer_insert(buffer, &iter, "Change bond distance/angle/dihedral angle on the spin button.\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
  else if (page == 17)
  {/*Change orbital type*/
    gtk_text_buffer_insert(buffer, &iter, "Open the \"orbitals\" notebook page and select the orbital you wish to change by clicking on it's row.\n\n", -1);

    gtk_text_buffer_insert(buffer, &iter, "Click on the first of the row (under orbital type). The menu should appear from which you can select a new type.\n\n", -1);
    do_link(buffer, &iter, "Go back", 3);
  }
}

static void follow_if_link(GtkWidget *text_view, GtkTextIter *iter)
{
  GSList *tags = NULL, *tagp = NULL;

  tags = gtk_text_iter_get_tags(iter);
  for (tagp = tags;  tagp != NULL;  tagp = tagp->next)
  {
    GtkTextTag *tag = tagp->data;
    gint page = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(tag), "page"));

    if(page != 0)
    {
      show_page(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view)), page);
      break;
    }
  }

  if (tags) 
  g_slist_free (tags);
}

static gboolean key_press_event(GtkWidget *text_view, GdkEventKey *event)
{
  GtkTextIter iter;
  GtkTextBuffer *buffer;

  switch (event->keyval)
  {
#ifdef GTK_OLD
    case GDK_Return: 
    case GDK_KP_Enter:
#else
    case GDK_KEY_Return:
    case GDK_KEY_KP_Enter:
#endif
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (text_view));
      gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
      follow_if_link(text_view, &iter);
      break;

    default:
      break;
  }

  return FALSE;
}

static gboolean visibility_notify_event(GtkWidget *text_view, GdkEventVisibility *event)
{
  gint wx, wy, bx, by;
  GSList *tags = NULL, *tagp = NULL;
  GtkTextIter iter;
  gboolean hovering = FALSE;

#ifdef GTK2
  gdk_window_get_pointer(text_view->window, &wx, &wy, NULL);
#endif
#ifdef GTK3
  GdkModifierType mask;
  printf("window get device position\n");
  gdk_window_get_device_position(gtk_widget_get_window(text_view), gtk_get_current_event_device(), &wx, &wy, &mask);
  printf("device position done\n");
#endif
  gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view), GTK_TEXT_WINDOW_WIDGET, wx, wy, &bx, &by);

  gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(text_view), &iter, bx, by);

  tags = gtk_text_iter_get_tags(&iter);

  for (tagp = tags; tagp != NULL; tagp = tagp->next)
  {
    GtkTextTag *tag = tagp->data;
    gint page = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(tag), "page"));

    if (page != 0) 
    {
      hovering = TRUE;
      break;
    }
  }

  if (hovering != hovering_over_link)
  {
    hovering_over_link = hovering;

    if (hovering_over_link)
      gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(text_view), GTK_TEXT_WINDOW_TEXT), hand_cursor);
    else
      gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(text_view), GTK_TEXT_WINDOW_TEXT), regular_cursor);
  }

  if (tags) g_slist_free(tags);


  return FALSE;
}

static gboolean event_after(GtkWidget *text_view, GdkEvent  *ev)
{
  GtkTextIter start, end, iter;
  GtkTextBuffer *buffer;
  GdkEventButton *event;
  gint x, y;

  if (ev->type != GDK_BUTTON_RELEASE) return FALSE;

  event = (GdkEventButton *)ev;

  if (event->button != 1) return FALSE;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));

  /* we shouldn't follow a link if the user has selected something */
  gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
  if (gtk_text_iter_get_offset(&start) != gtk_text_iter_get_offset(&end)) return FALSE;

  gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW (text_view), GTK_TEXT_WINDOW_WIDGET, event->x, event->y, &x, &y);

  gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(text_view), &iter, x, y);

  follow_if_link(text_view, &iter);

  return FALSE;
}

void kill_help_window(GtkWidget *widget, GtkWidget *help_window)
{
  gtk_widget_destroy(help_window);
}

void luscus_gtk_callback_help(GtkWidget *widget, gpointer data)
{
  GtkWidget *help_window;
  GtkWidget *button;
  GtkWidget *vbox;
  GtkWidget *separator;
  GtkWidget *text_view;
  GtkWidget *scrolled_window;
  GtkTextBuffer *tbuff;

  hand_cursor = gdk_cursor_new (GDK_HAND2);
  regular_cursor = gdk_cursor_new (GDK_XTERM);

  help_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  g_signal_connect(G_OBJECT(help_window), "delete-event", G_CALLBACK(kill_help_window), (gpointer) help_window);
  
#ifdef GTK2
  vbox = gtk_vbox_new(FALSE, 0);
#endif
#ifdef GTK3
  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
  gtk_box_set_homogeneous(GTK_BOX(vbox), FALSE);  
#endif
  gtk_container_add(GTK_CONTAINER(help_window), vbox);

  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  text_view = gtk_text_view_new();
  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_WORD);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);
  gtk_container_add(GTK_CONTAINER(scrolled_window), text_view);
  gtk_widget_set_size_request(GTK_WIDGET(text_view), 600, 450);
  g_signal_connect(text_view, "key-press-event", G_CALLBACK(key_press_event), NULL);
  g_signal_connect(text_view, "visibility-notify-event", G_CALLBACK(visibility_notify_event), NULL);
  g_signal_connect(text_view, "event-after",  G_CALLBACK(event_after), NULL);

  gtk_widget_show(text_view);
  gtk_widget_show(scrolled_window);

  tbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
  show_page(tbuff, 1);

/*  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
  gtk_widget_show(separator);*/

  button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(kill_help_window), (gpointer) help_window);
  gtk_widget_show(button);

  gtk_widget_show(vbox);
  gtk_widget_show(help_window);
}

